@@ -1,4 +1,5 @@
 import asyncio
+from contextlib import asynccontextmanager
 
 from ..pool import ConnectionPool, ClosedPool, EmptyPool
 
@@ -9,22 +10,6 @@ MYPY = False
 if MYPY:
     from ..ldapclient import LDAPClient
 
-
-class AIOPoolContextManager:
-    def __init__(self, pool, *args, **kwargs):
-        self.pool = pool
-        self.__conn = None
-
-    async def __aenter__(self):
-        if self.pool.closed:
-            await self.pool.open()
-        self.__conn = await self.pool.get()
-        return self.__conn
-
-    async def __aexit__(self, type, value, traceback):
-        await self.pool.put(self.__conn)
-
-
 class AIOConnectionPool(ConnectionPool):
     """
     A connection pool that can be used with asnycio tasks. It's inherited from
@@ -97,5 +82,12 @@ class AIOConnectionPool(ConnectionPool):
             super().close()
             self._lock.notify_all()
 
-    def spawn(self, *args, **kwargs):
-        return AIOPoolContextManager(self, *args, **kwargs)
+    @asynccontextmanager
+    async def spawn(self, *args, **kwargs):
+        try:
+            if self._closed:
+                await self.open()
+            conn = await self.get(*args, **kwargs)
+            yield conn
+        finally:
+            await self.put(conn)
